/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Agent Management

'use strict';

var path = require("path");

var evt = require("./evt");
var env = require("./env");
var log = require("./log");
var Agent = require('./cls/agent');

//var AGENT_NAME = env.ifMac("iSedoraAgent.app/Contents/MacOS/iSedoraAgent", "iSedoraAgent.exe");
var AGENT_NAME = env.ifMac("iSedoraAgent", "iSedoraAgent.exe");
var AGENT_PATH = path.join(process.env['ISEDORA_BIN'] || path.dirname(process.execPath), AGENT_NAME);
var AGENT_ARGS = ["-confirmed"];

var agents = [];

//-----------------------------------------------------------------------------

function firstAgentReady() {
  log.debug('Agent ready');
  setImmediate(evt.postAgentReady());
}

function emptyCallback(err) {
  if (err) {
    log.error('Agent response error: %s', err);
  }
}

//-----------------------------------------------------------------------------

exports.start = function() {

  if (agents.length > 0) {
    throw new Error('Agent already started');
  }

  var agt = new Agent(AGENT_PATH, AGENT_ARGS);

  agents.push(agt);

  agt.once('ready', firstAgentReady);
  agt.on('exit', onexit);

  agt.start();

};

//-----------------------------------------------------------------------------

var lastAgent = 0;

function getLessBusyAgent() {
  var alen = agents.length;
  if (alen === 0) throw "No Agents running";
  if (alen === 1) return agents[0];
  if (lastAgent >= alen) lastAgent = 0;
  return agents[lastAgent++];
}

//-----------------------------------------------------------------------------

function send(name, cb, args, agtindex) {

  if (agents.length === 0) {
    throw new Error('No agent running');
  }

  if (!cb) cb = emptyCallback;

  var data = {"name": name};

  if (args !== undefined) {
    data.args = args;
  }

  var agt = (agtindex >= 0) ? agents[agtindex] : getLessBusyAgent();

  agt.send(data, cb);

}

//-----------------------------------------------------------------------------

function onready(agt) {
  agents.push(agt);
  log.trace('New agent ready: %s (pool size: %d)', agt, agents.length);
}

function onexit(agt) {
  var ix = agents.indexOf(agt);
  if (ix > -1) {
    log.trace('Removing agent: %s', agt);
    agents.splice(ix,  1);
  }
  if (agents.length === 0 && !evt.exiting()) {
    log.error('No agent is running');
    evt.addExitProc(function(cb) {env.werr('No agent is running', cb)});
    evt.quit(100);
  }
}

//-----------------------------------------------------------------------------

function onconfig() {
  var cfg = require('./cfg');
  var agentCount = cfg.agentCount();
  if (typeof agentCount !== 'number') {
    log.error('Config agentCount is not a number: %s', agentCount);
    return;
  }
  if (agentCount < 1 || agentCount > 16) {
    log.error('Insane agent count: %d', agentCount);
    return;
  }
  var alen = agents.length;
  var agt;
  if (agentCount > alen) {
    while(agentCount > alen) {
      agt = new Agent(AGENT_PATH, AGENT_ARGS);
      agt.on('exit', onexit);
      agt.on('ready', onready);
      agt.start();
      alen++;
    }
  }
  else if (agentCount < alen) {
    while(agentCount < alen) {
      agt = agents.pop();
      agt.interrupt();
      alen--;
    }
  }
}

evt.onConfigReadyChange(onconfig);

//-----------------------------------------------------------------------------
// Agent Interface
//-----------------------------------------------------------------------------

exports.sysinfo = function(cb) {
  send("sysinfo", cb);
};

exports.formats = function(cb) {
  send("formats", cb);
};

exports.broadcastReady = function(cb) {
  send("broadcast", cb, {name: 'com.isedora.ready'});
};

exports.minfo = function(fpath, index, cb) {
  send("minfo", cb, {fpath: fpath, index: index});
};

exports.plist = function(fpath, cb) {
  send("plist", cb, {path: fpath});
};

exports.mkfifo = function(fpath, cb) {
  send("mkfifo", cb, {path: fpath});
};

exports.pm_deny = function(cb) {
  send('pm_deny', cb, null, 0);
};

exports.pm_allow = function(cb) {
  send('pm_allow', cb, null, 0);
};

exports.pm_reset = function(cb) {
  send('pm_reset', cb, null, 0);
};

exports.scale = function(spath, tpath, ftype, mtype, index, width, height, bcolor, orn, cb) {
  send("scale", cb, {
    spath:  spath,
    tpath:  tpath,
    ftype:  ftype,
    mtype:  mtype,
    index:  index,
    width:  width,
    height: height,
    bcolor: bcolor,
    orn: orn
  });
};

exports.agentPath = function() { return AGENT_PATH };