/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var bodyParser = require('body-parser');
var request = require('request');
var nurl = require('url');

var upnp = require('./upnp');

var log  = require('../log');
var rti  = require('../rti');
var xml  = require('../xml');
var htp  = require('../htp');
var md5  = require('../md5');
var evt  = require('../evt');

var id = rti.did;

var SND_FILE_OPT  = { etag: false };

function Service(devPath, type, scid, scpd) {

  var self = this;

  var nary = type.split(':');
  var nlen = nary.length;

  var fullname = nary[nlen - 2] + nary[nlen - 1];

  var uctl = '/ctl/' + fullname;
  var uevt = '/evt/' + fullname;
  var uscp = '/' + fullname + '.xml';

  var emap = {}; // events map
  var esub = {}; // events subscriptions

  setInterval(cleanEvents, 1000 * 60);

  self[id.serviceType] = type;

  self.event = function(name, value) {
    emap[name] = value;
  };

  //-----------------------------------------------------------------------------

  function sendFault(req, res, err) {
    var action = req.get('SOAPAction');
    if (action && action.indexOf('#') > 0) {
      action = action.replace(/"/g, '');
      action = fullname + '::' + action.split('#')[1];
    }
    else {
      action = fullname;
    }
    res.status(500).send(xml.createUpnpFault(err));
    log.error('%s: %s', action, err);
  }

  //-----------------------------------------------------------------------------

  function onSoap(req, res) {

    htp.setXmlMime(res);

    res.set('Ext', '');

    var body = req.body;

    if (!body) {
      sendFault(req, res, 'Empty SOAP body');
      return;
    }

    var arg = {};
    var act = xml.parseSoapAction(body, arg);

    if (!act) {
      sendFault(req, res, 'SOAP action not found in request body');
      return;
    }

    if (log.canSoap()) {
      log.soap('REQ %s::%s %s', fullname, act, log.stringify(arg));
    }

    evt.postSoapRequest(req, act, arg);

    var proc = self[act];

    if (typeof proc !== 'function') {
      sendFault(req, res, 'SOAP action not implemented');
      return;
    }

    proc(req, arg, function(err, obj) {
      if (err) {
        return sendFault(req, res, err);
      }
      if (obj === undefined) obj = null;
      if (log.canSoap()) {
        log.soap('RES %s::%s %s', fullname, act, log.stringify(obj));
      }
      var soapXml = xml.createSoapResponse(act, type, obj);
      res.send(soapXml);
    });

  }

  //-----------------------------------------------------------------------------

  function sendEventToAll(obj) {
    for (var sid in esub) {
      var evt = esub[sid];
      log.http('Sending event to %s', evt.cback);
      sendEvent(sid, esub[sid], obj);
    }
  }

  self.sendEventToAll = sendEventToAll;

  //-----------------------------------------------------------------------------

  function sendInitialEvent(sid) {
    var evt = esub[sid];
    if (!evt) return;
    log.http('%s sending initial event to %s', fullname, evt.cback);
    sendEvent(sid, evt, emap);
  }

  function removeSid(sid) {
    delete esub[sid];
  }

  function sendEvent(sid, evt, obj) {
    var opts = {
      method: 'NOTIFY',
      url: evt.cback,
      headers: {
        Host: evt.host,
        'Content-Type': 'text/xml',
        NT: 'upnp:event',
        NTS: 'upnp:propchange',
        SID: sid,
        SEQ: evt.seq
      },
      body: xml.eventProperties(obj, self),
      timeout: 5000
    };
    evt.seq++;
    request(opts, function(err, res, body) {
      if (err) {
        log.error('Event send error to %s (%s)', evt.cback, err);
        setImmediate(removeSid, sid);
        return;
      }
      if (res.statusCode !== 200) {
        log.error('Event send status error to %s (%d)', evt.cback, res.statusCode);
        setImmediate(removeSid, sid);
        return;
      }
      log.http('Event sent success to %s %s', fullname, evt.cback);
    });
  }

  function cleanEvents() {
    var now = Date.now();
    var off = [];
    for (var sid in esub) {
      var evt = esub[sid];
      var tout = evt.tout;
      if (tout && (evt.stamp + (tout * 1000)) < now) {
        off.push(sid);
      }
    }
    for (var i = 0; i < off.length; i++) {
      sid = off[i];
      log.http('Removing expired subscription %s', sid);
      delete esub[sid];
    }
  }

  //-----------------------------------------------------------------------------

  function onEvent(req, res) {

    var method = req.method;
    var sid = req.get('sid');
    var timeout = req.get('timeout');

    res.set('Date', new Date().toUTCString());

    if (method === 'UNSUBSCRIBE') {
      if (sid) {
        log.http('Removing event subscription %s %s', fullname, sid);
        delete esub[sid];
      }
      res.status(200).end();
      return;
    }

    if (method === 'SUBSCRIBE') {
      var tsecond = 0;
      if (timeout && timeout !== 'infinite' && timeout.indexOf('-') > 0) {
        tsecond = parseInt(timeout.split('-')[1]);
        if (isNaN(tsecond)) tsecond = 0;
      }
      if (!sid) {
        sid = 'uuid:' + md5.uuid();
        var cback = req.get('callback');
        if (cback) {
          cback = cback.replace(/<|>/g, '');
          log.http('New event subscription %s %s', fullname, cback);
          var host = nurl.parse(cback);
          esub[sid] = {cback: cback, seq: 0, host: host.host, stamp: Date.now(), tout: tsecond};
          setTimeout(function() {
            sendInitialEvent(sid);
          }, 100);
        }
      }
      else {
        var evt = esub[sid];
        if (evt) {
          log.http('Update event subscription %s %s', fullname, evt.cback);
          evt.stamp = Date.now();
          evt.tout = tsecond;
        }
      }
      res.set('SID', sid);
      res.set('Timeout', timeout);
      res.status(200).end();
      return;
    }

    res.status(500).end();

  }

  //-----------------------------------------------------------------------------

  function onScpd(req, res) {
    htp.setXmlMime(res);
    res.sendFile(scpd, SND_FILE_OPT);
  }

  //-----------------------------------------------------------------------------

  self.initRoute = function(rout) {
    rout.use(uscp, onScpd);
    rout.use(uevt, onEvent);
    rout.use(uctl, bodyParser.text({type: 'text/xml'}));
    rout.use(uctl, onSoap);
  };

  //-----------------------------------------------------------------------------

  self.description = function(url) {
    return {
      service: {
        serviceType: type,
        serviceId:   scid,
        SCPDURL:     url + uscp,
        controlURL:  url + uctl,
        eventSubURL: url + uevt
      }
    };
  };

}

exports.Service = Service;