/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Info Handling

'use strict';

var fs   = require('fs');
var mime = require('mime');

var log = require('../log');
var agt = require('../agt');
var env = require('../env');
var md5 = require('../md5');
var cfg = require('../cfg');
var mdb = require('./mdb');
var alg = require('./alg');
var rti = require('../rti');
var evt = require('../evt');
var jsl = require('../jsl');
var prf = require('./prf');

var id    = rti.id;
var ftype = rti.ftype;
var mtype = rti.mtype;

var minfo = mdb.minfo;

var BIN_MIME   = 'application/octet-stream';

var THUMB_DLNA  = 160;

var mmap = {
  'mpeg-ps': prf.mime.mpeg,
  'mpeg-ts': prf.mime.mpts,
  'bdav':    prf.mime.mpts,
  'adts':    prf.mime.adts,
  'flac':    prf.mime.flac,
  'm4a':     prf.mime.mp4a,
  'wave':    prf.mime.wav,
  'divx':    prf.mime.avi
};

//-----------------------------------------------------------------------------

function emptyData(uid, fpath) {
  var md = {};
  md[id.uid]   = uid;
  md[id.fpath] = fpath;
  md[id.ftype] = ftype.unknown;
  md[id.mtype] = mtype.unknown;
  md[id.stamp] = Date.now();
  setThumbDimension(md);
  return md;
}

//-----------------------------------------------------------------------------

function fileInfo(fpath, index, cache, cb) {

  var uid = md5(fpath);

  if (index > 0) uid += '_' + index;

  if (cache) {
    var md = minfo.get(uid);
    if (md) {
      return cb(null, md);
    }
  }

  agt.minfo(fpath, index, function(err, md) {

    if (err) {
      log.error('minfo error: %s', err);
      var zmd = emptyData(uid, fpath);
      zmd[id.error] = err.toString();
      minfo.set(uid, zmd);
      return cb(null, zmd);
    }

    md[id.uid] = uid;

    setThumbDimension(md);

    if (md[id.ftype] === ftype.unknown) {
      minfo.set(uid, md);
      return cb(null, md);
    }

    if (rti.isFile(md)) {
      if (md[id.fextn]) {
        md[id.fmime] = mime.lookup(md[id.fextn]);
      }
      else {
        md[id.fmime] = BIN_MIME;
      }
      avparse(md);
      if (md[id.mtype] === mtype.unknown) {
        guessMediaType(md);
      }
      var fmt = md[id.format];
      if (fmt !== undefined) {
        var mim = mmap[fmt];
        if (mim !== undefined) {
          md[id.fmime] = mim;
        }
      }
    }

    md[id.stamp] = Date.now();

    minfo.set(uid, md);

    cb(null, md);

  });

}

exports.fileInfo = fileInfo;

//-----------------------------------------------------------------------------

function setThumbDimension(md) {

  md[id.tvalid] = true;

  if (rti.isImageOrPDF(md)) {
    var w = md[id.width];
    var h = md[id.height];
    if (w > 0 && h > 0) {
      var tw = THUMB_DLNA;
      var th = THUMB_DLNA;
      if (w > tw || h > th) {
        var sz = alg.scale(w, h, tw, th);
        tw = sz[0];
        th = sz[1];
      }
      else {
        tw = w;
        th = h;
      }
      md[id.twidth] = tw;
      md[id.theight] = th;
    }
    else {
      md[id.twidth]  = THUMB_DLNA;
      md[id.theight] = THUMB_DLNA;
    }
  }
  else {
    var tsize = THUMB_DLNA;
    md[id.twidth]  = tsize;
    md[id.theight] = tsize;
  }

}

//-----------------------------------------------------------------------------


function guessMediaType(md) {

  var vc = rti.videoCount(md);
  var ac = rti.audioCount(md);
  var tc = rti.textCount(md);

  if (vc > 0) vc = checkTracks(md[id.video], mtype.video);
  if (ac > 0) ac = checkTracks(md[id.audio], mtype.audio);
  if (tc > 0) tc = checkTracks(md[id.text], mtype.text);

  if (tc > 0 && vc === 0 && ac === 0) {
    md[id.mtype] = mtype.text;
    return;
  }

  if (ac > 0) {
    if (vc === 0 || jsl.hasPrefix(rti.fmime(md), 'audio/')) {
      md[id.mtype] = mtype.audio;
      return;
    }
  }

  if (vc > 0) {
    md[id.mtype] = mtype.video;
  }

}

function checkTracks(ary, type) {
  var cnt = ary.length;
  var len = cnt;
  for (var i = 0; i < len; i++) {
    var trk = ary[i];
    var valid = true;
    switch (type) {
      case mtype.audio:
        if (rti.isNull(trk[id.codec]) || !rti.isNumber(trk[id.samplingrate]) || !rti.isNumber(trk[id.channels])) {
          valid = false;
        }
        break;
      case mtype.video:
        if (rti.isNull(trk[id.codec]) || !rti.isNumber(trk[id.width])) {
          valid = false;
        }
        break;
      case mtype.text:
        if (rti.isNull(trk[id.codec])) {
          valid = false;
        }
        break;
    }
    if (!valid) cnt--;
    trk[id.valid] = valid;
  }
  return cnt;
}

//-----------------------------------------------------------------------------

function avparse(md) {

  var minfo = md[id.minfo];

  if (!minfo) return;

  delete md[id.minfo];

  var lines = minfo.split("\n");
  var count = lines.length;
  var obj, key, val, pos, str;

  for (var i = 0; i < count; i++) {
    str = lines[i].trim();
    if (str.length === 0) {
      obj = null;
      continue;
    }
    pos = str.indexOf(',');
    if (pos > 0) {
      key = str.substring(0, pos);
      val = str.substring(pos + 1);
    }
    else {
      key = str;
      val = null;
    }
    switch (key) {
      case 'General':
        obj = md;
        break;
      case 'Video':
        obj = {};
        addTrack(md, id.video, obj);
        break;
      case 'Audio':
        obj = {};
        addTrack(md, id.audio, obj);
        break;
      case 'Text':
        obj = {};
        addTrack(md, id.text, obj);
        break;
      case 'Image':
      case 'Menu':
      case 'Other':
        break;
      default:
        if (obj) {
          addAV(obj, key, val);
        }
    }
  }

}

//-----------------------------------------------------------------------------

function addTrack(md, name, track) {
  var ary = md[name];
  if (!ary) {
    ary = [];
    md[name] = ary;
  }
  ary.push(track);
  track[id.streamorder] = ary.length - 1;
}

//-----------------------------------------------------------------------------

var AVGOOD = {};

var AVKEYS = {
  'channel(s)': 'channels',
  overallbitrate: id.bitrate,
  overallbitrate_mode: id.bitmode,
  bitrate_mode: id.bitmode
};

//-----------------------------------------------------------------------------

function addAV(obj, key, val) {
  key = key.toLowerCase();
  if (AVGOOD[key] !== true || val === '(Binary)') return;
  key = avkey(key);
  if (likelyNumber(val)) {
    var num = (val.indexOf('.') > 0) ? parseFloat(val) : parseInt(val);
    if (Number.isFinite(num)) {
      if (key === id.codecid) {
        return;
      }
      if (key === id.duration) {
        num = Math.round(num);
      }
      obj[key] = num;
      return;
    }
  }
  switch (key) {
    case 'format':
    case 'codec':
      val = val.toLowerCase();
      if (val === 'quicktime') val = 'qt';
      if (obj[key] === undefined) obj[key] = val;
      return;
    case 'codecid':
      var pos = val.indexOf('/');
      if (pos > 0) val = val.substring(0, pos).trim();
      val = val.toLowerCase();
      obj[key] = val;
      if (val === 'qt') {
        obj[id.codec] = 'qt';
      }
      return;
    case id.language:
      obj[key] = [val];
      return;
  }
  if (jsl.hasPrefix(key, 'language/string')) {
    val = val.toLowerCase();
    var lang = obj[id.language];
    if (!Array.isArray(lang)) {
      lang = [];
      obj[id.language] = lang;
    }
    if (lang.indexOf(val) === -1) {
      lang.push(val);
    }
    return;
  }
  obj[key] = val;
}

//-----------------------------------------------------------------------------

function avkey(key) {
  return AVKEYS[key] || key;
}

//-----------------------------------------------------------------------------

function likelyNumber(val) {
  switch (val.charAt(0)) {
    case '-':
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
      return true;
  }
  return false;
}

//-----------------------------------------------------------------------------

function initGoodKeys() {

  var ary = [
    'format', 'codec', 'codecid', 'duration', 'framerate', 'title', 'internetmediatype',
    'bitrate', 'overallbitrate', 'overallbitrate_mode', 'bitrate_mode',
    'isstreamable', 'codec_profile', 'scantype',
    'album', 'track', 'track/position', 'track/position_total', 'performer', 'genre', 'movie', 'part', 'rating',
    'season',
//    'streamorder',
    'id',
    'channel(s)', 'samplingrate', 'compression_mode', 'channelpositions', 'channellayout',
    'resolution', 'bitdepth',
    'width', 'height', 'pixelaspectratio', 'displayaspectratio', 'rotation', 'framerate_mode',
    'resolution', 'colorspace',
    'language', 'language/string1', 'language/string2', 'language/string3', 'language/string4',
    'language/string5'
  ];

  for (var i = 0; i < ary.length; i++) {
    AVGOOD[ary[i]] = true;
  }

}

//-----------------------------------------------------------------------------

initGoodKeys();



