/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Top Folders Handler

'use strict';

var async = require('async');

var hnd = require('../hnd');
var cfg = require('../cfg');
var rti = require('../rti');
var msg = require('../msg');
var md5 = require('../md5');
var fop = require('../fop');
var didl = require('../cls/didl');
var upnp = require('../cls/upnp');

var mdi = require('./mdi');
var mdb = require('./mdb');

var mtype = rti.mtype;
var Container = didl.Container;

//-----------------------------------------------------------------------------

function rootHandler(brq, cb) {

  if (brq.meta) {
    var root = new Container(hnd.ROOT, '-1', cfg.serverName(), countRoot(brq));
    root.setUpnpClass(upnp.STORAGE_SYSTEM_CONTAINER);
    root.setHideCount(true);
    brq.ary.push(root);
    return cb();
  }

  var topOrder = cfg.topOrder();

  if (cfg.topGroup()) {
    for (var i = 0; i < topOrder.length; i++) {
      rootGroup(brq, topOrder[i])
    }
    return cb();
  }

  async.eachSeries(topOrder, function(mt, xcb) { rootFlat(brq, mt, xcb) }, cb);

}

hnd.addRootBrowser(rootHandler);

//-----------------------------------------------------------------------------

function rootFlat(brq, mt, cb) {
  var hname;
  switch (mt) {
    case mtype.image:
      hname = hnd.PHOTO;
      break;
    case mtype.audio:
      hname = hnd.MUSIC;
      break;
    case mtype.video:
      hname = hnd.VIDEO;
      break;
    default:
      return cb();
  }
  hnd.runBrowseHandlers(hname, brq, false, cb);
}

//-----------------------------------------------------------------------------

function rootGroup(brq, mt) {
  if (mt === mtype.image) {
    if (cfg.enableImage()) {
      var p = new Container(hnd.PHOTO, hnd.ROOT, msg('Photo'), countPhoto(brq));
      p.setMediaClass(upnp.AV_PHOTO);
      brq.ary.push(p);
    }
  }
  else if (mt === mtype.audio) {
    if (cfg.enableAudio()) {
      var m = new Container(hnd.MUSIC, hnd.ROOT, msg('Music'), countMusic(brq));
      m.setMediaClass(upnp.AV_MUSIC);
      brq.ary.push(m);
    }
  }
  else if (mt === mtype.video) {
    if (cfg.enableVideo()) {
      var v = new Container(hnd.VIDEO, hnd.ROOT, msg('Video'), countVideo(brq));
      v.setMediaClass(upnp.AV_VIDEO);
      brq.ary.push(v);
    }
  }
}

//-----------------------------------------------------------------------------

function addMediaRoots(brq, mt, mediaClass, roots, cb) {

  var countProc = (cfg.childCount()) ? fop.childCount : fop.dummyCount;

  var pary = Object.keys(roots);

  var plen = pary.length;

  if (plen === 0) return cb();

  var hideEmpty = cfg.hideEmpty();

  async.map(pary, countProc, function(err, counts) {

    for (var i = 0; i < plen; i++) {
      pary[i] = {path: pary[i], count: counts[i]};
    }

    async.map(pary,
      function(pc, xcb) {
        var path  = pc.path;
        var count = pc.count;
        if (hideEmpty && count === 0) return xcb(null, null);
        mdi.fileInfo(path, 0, true, function(err, md) {
          if (!rti.isFolder(md)) return xcb(null, null);
          var uid = rti.uid(md);
          var top = roots[path];
          mdb.parents.set(uid, null);
          var cnt = new Container(hnd.fsoid(mt, uid), brq.oid, top.name, count);
          cnt.setMediaClass(mediaClass);
          xcb(null, cnt);
        });
      },
      function(err, dtop) {
        var dlen = dtop.length;
        for (var i = 0; i < dlen; i++) {
          var td = dtop[i];
          if (td) brq.ary.push(td);
        }
        cb();
      }
    );

  });

}

//-----------------------------------------------------------------------------

function photoHandler(brq, cb) {
  if (!cfg.enableImage()) return cb();
  if (brq.meta) {
    rootGroup(brq, mtype.image);
    return cb();
  }
  addMediaRoots(brq, mtype.image, upnp.AV_PHOTO, cfg.imageRoots(), cb);
}

hnd.addPhotoBrowser(photoHandler);

//-----------------------------------------------------------------------------

function musicHandler(brq, cb) {
  if (!cfg.enableAudio()) return cb();
  if (brq.meta) {
    rootGroup(brq, mtype.audio);
    return cb();
  }
  addMediaRoots(brq, mtype.audio, upnp.AV_MUSIC, cfg.audioRoots(), cb);
}

hnd.addMusicBrowser(musicHandler);

//-----------------------------------------------------------------------------

function videoHandler(brq, cb) {
  if (!cfg.enableVideo()) return cb();
  if (brq.meta) {
    rootGroup(brq, mtype.video);
    return cb();
  }
  addMediaRoots(brq, mtype.video, upnp.AV_VIDEO, cfg.videoRoots(), cb);
}

hnd.addVideoBrowser(videoHandler);

//-----------------------------------------------------------------------------

function countRoot(brq) {
  var rv = hnd.count(brq, hnd.ROOT);
  if (cfg.topGroup()) {
    if (cfg.enableImage()) rv++;
    if (cfg.enableAudio()) rv++;
    if (cfg.enableVideo()) rv++;
  }
  else {
    if (cfg.enableImage()) rv += countPhoto(brq);
    if (cfg.enableAudio()) rv += countMusic(brq);
    if (cfg.enableVideo()) rv += countVideo(brq);
  }
  return rv;
}

function countPhoto(brq) {
  return Object.keys(cfg.imageRoots()).length + hnd.count(brq, hnd.PHOTO);
}

function countMusic(brq) {
  return Object.keys(cfg.audioRoots()).length + hnd.count(brq, hnd.MUSIC);
}

function countVideo(brq) {
  return Object.keys(cfg.videoRoots()).length + hnd.count(brq, hnd.VIDEO);
}
