/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

//------------------
// CONTENT DIRECTORY
//------------------

'use strict';

var npath = require('path');
var util  = require('util');

var evt = require("../evt");
var env = require("../env");
var cfg = require("../cfg");
var log = require("../log");
var rti = require('../rti');
var hnd = require('../hnd');
var htp = require('../htp');
var jsl = require('../jsl');
var sid = require('./sysid');
var dev = require('../ssdp/dev');

var upnp = require('../cls/upnp');
var didl = require('../cls/didl');

var Service = require('../cls/service').Service;
var BrowseRequest = require('../cls/hrequest').BrowseRequest;

var id = rti.did;

var SERVICE_TYPE = 'urn:schemas-upnp-org:service:ContentDirectory:1';
var SERVICE_ID   = 'urn:upnp-org:serviceId:ContentDirectory';
var SERVICE_SCPD = npath.join(global['ISEDORA_DIR'], 'pub', 'UPnP_AV_ContentDirectory_1.0.xml');

//-----------------------------------------------------------------------------
// Constructor
//-----------------------------------------------------------------------------

function ContentDirectory(devPath) {

  Service.call(this, devPath, SERVICE_TYPE, SERVICE_ID, SERVICE_SCPD);

  var self = this;

  var streamPath = '/s';

  self.event('SystemUpdateID', sid.getSysID);
  self.event('TransferIDs', null);
  self.event('ContainerUpdateIDs', null);

  sid.onChange(function(sysid) {
    self.sendEventToAll({SystemUpdateID: sysid});
  });

  //-----------------------------------------------------------------------------

  self.GetSystemUpdateID = function(req, arg, cb) {
    cb(null, {Id: sid.getSysID()});
  };

  //-----------------------------------------------------------------------------

  self.GetSortCapabilities = function(req, arg, cb) {
    cb(null, {SortCaps: '*'});
  };

  //-----------------------------------------------------------------------------

  self.GetSearchCapabilities = function(req, arg, cb) {
    cb(null, {SearchCaps: '*'});
  };

  //-----------------------------------------------------------------------------

  self.Search = function(req, arg, cb) {
    cb('Action not implemented');
  };

  //-----------------------------------------------------------------------------

  self.Browse = function(req, arg, cb) {

    evt.postBeforeBrowseRequest(req, arg);

    var oid = arg['objectid'];
    var flag = arg['browseflag'];
    var filter = arg['filter'];
    var start = parseInt(arg['startingindex'], 10);
    var count = parseInt(arg['requestedcount'], 10);
    var sort  = arg['sortcriteria'];

    if (isNaN(start)) start = 0;
    if (isNaN(count) || count === 0) count = 10000;

    var surl = htp.httpAddress(req) + devPath + streamPath;

    var meta = (flag === 'BrowseMetadata');

    var dcfg = dev.getDeviceConfig(req.ip);

    var brq = new BrowseRequest(dcfg, oid, req, meta, surl, start, count);

    hnd.browseRequest(brq, function(err) {

      if (err) {
        return cb(err);
      }

      evt.postBrowseRequest(brq);

      var rv = {
        Result: null,
        NumberReturned: 1,
        TotalMatches: 1,
        UpdateID: sid.getSysID()
      };

      var ary = brq.ary;
      var len = ary.length;

      if (brq.meta) {
        if (len === 0) {
          return cb('Browse MetaData not found for: ' + oid);
        }
        if (len > 1) {
          return cb('Browse MetaData returned more than one object for: ' + oid);
        }
        rv.Result = didl.didlXML(ary);
        return cb(null, rv);
      }

      if (len === 0) {
        rv.Result = didl.didlXML(ary);
        rv.TotalMatches = 0;
        rv.NumberReturned = 0;
        return cb(null, rv);
      }

      if (brq.range) {
        rv.Result = didl.didlXML(ary);
        rv.TotalMatches = brq.total;
        rv.NumberReturned = ary.length;
        return cb(null, rv);
      }

      var range = [];

      for (var i = start; i < (start + count) && i < len; i++) {
        range.push(ary[i]);
      }

      rv.Result = didl.didlXML(range);
      rv.TotalMatches = len;
      rv.NumberReturned = range.length;

      cb(null, rv);

    });

  };

  //-----------------------------------------------------------------------------

  function initRoute(rout) {
    hnd.initStreamRouter(rout, streamPath);
  }

  jsl.overload(self, 'initRoute', initRoute);

}

util.inherits(ContentDirectory, Service);

//-----------------------------------------------------------------------------
// Event
//-----------------------------------------------------------------------------

evt.onRootDeviceInit(function(dev) {
  if (dev[id.deviceType] === upnp.DMS1) {
    dev.addService(new ContentDirectory(dev.path));
  }
});



