/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// X-AV-Physical-Unit-Info: pa="Blu-ray Disc Player";
// X-AV-Client-Info: av=5.0; cn="Sony Corporation"; mn="Blu-ray Disc Player"; mv="2.0";

var fs  = require('fs');
var log = require('./log');
var rti = require('./rti');
var fmt = require('./fmt');

var XAV_UNIT_INFO   = 'X-AV-PHYSICAL-UNIT-INFO';
var XAV_CLIENT_INFO = 'X-AV-CLIENT-INFO';

var XML_MIME = 'text/xml; charset=UTF-8';
var TXT_MIME = 'text/plain; charset=UTF-8';

exports.XML_MIME = XML_MIME;
exports.TXT_MIME = TXT_MIME;

var GET_CONTENT_FEATURES_DLNA_ORG = "getcontentfeatures.dlna.org";
var CONTENT_FEATURES_DLNA_ORG     = "contentFeatures.dlna.org";
var TRANSFER_MODE                 = 'TransferMode.DLNA.ORG';
var TRANSFER_INTERACTIVE          = 'Interactive';
var TRANSFER_STREAMING            = 'Streaming';
var TIMESEEK_RANGE                = 'TimeSeekRange.dlna.org';
var XSEEK_RANGE                   = 'X-Seek-Range';

exports.TRANSFER_INTERACTIVE = TRANSFER_INTERACTIVE;
exports.TRANSFER_STREAMING   = TRANSFER_STREAMING;

exports.SEND_FILE_OPT = function() { return { etag: false } };

exports.setXmlMime = function(res) {
  res.setHeader('Content-Type', XML_MIME);
};

exports.setTextMime = function(res) {
  res.setHeader('Content-Type', TXT_MIME);
};

exports.setContentType = function(res, contentType) {
  res.setHeader('Content-Type', contentType);
};

//-----------------------------------------------------------------------------

function parseXAV(header) {
  var map = {};
  var ary = header.split(';');
  for (var i = 0; i < ary.length; i++) {
    var kv = ary[i].trim().split('=');
    if (kv.length === 2) {
      var key = kv[0].trim();
      var val = kv[1].trim();
      val = val.replace(/"/g, '');
      if (key.length && val.length) {
        map[key] = val;
      }
    }
  }
  return map;
}

exports.getXAV = function(req) {
  var unitInfo = req.get(XAV_UNIT_INFO);
  var clntInfo = req.get(XAV_CLIENT_INFO);
  if (!unitInfo && !clntInfo) {
    return null;
  }
  var friendlyName;
  var manufacturer;
  if (unitInfo) {
    var umap = parseXAV(unitInfo);
    friendlyName = umap['pa'];
  }
  if (clntInfo) {
    var cmap = parseXAV(clntInfo);
    if (!friendlyName) {
      friendlyName = cmap['mn'];
    }
    manufacturer = cmap['cn'];
  }
  if (friendlyName || manufacturer) {
    var hmap = {};
    if (friendlyName) {
      hmap.friendlyName = friendlyName;
    }
    if (manufacturer) {
      hmap.manufacturer = manufacturer;
    }
    return hmap;
  }
  return null;
};

//-----------------------------------------------------------------------------

exports.httpAddress = function(req) {
  var addr = req.socket.address();
  return 'http://' + addr.address + ':' + addr.port;
};

exports.userAgent = function(req) {
  return req.get('user-agent');
};

//-----------------------------------------------------------------------------
// Transfer Mode
//-----------------------------------------------------------------------------

function setTransferMode(res, mode) {
  res.set(TRANSFER_MODE, mode);
}

exports.setTransferMode = setTransferMode;

function setDlnaHeaders(srq, transferMode) {
  var req = srq.req;
  var res = srq.res;
  if (transferMode) {
    res.set(TRANSFER_MODE, transferMode);
  }
  if (req.get(GET_CONTENT_FEATURES_DLNA_ORG)) {
    if (srq.prf) {
      res.set(CONTENT_FEATURES_DLNA_ORG, srq.prf);
    }
    else {
      log.warn('Stream request has no profile: %s', srq.req.originalUrl);
    }
  }
}

exports.setDlnaHeaders = setDlnaHeaders;

//-----------------------------------------------------------------------------
// Time Range Functions
// TimeSeekRange.dlna.org: npt=0.000-
// TimeSeekRange.dlna.org: npt=00:00:00-00:03:08/00:03:08
// TimeSeekRange.dlna.org: npt=00:00:00.000-00:03:08/00:03:08
//-----------------------------------------------------------------------------

function getTimeRange(req, duration) {

  if (!rti.isNumber(duration)) {
    return null;
  }

  var tr = req.header(TIMESEEK_RANGE);

  if (tr === undefined) {
    tr = req.header(XSEEK_RANGE);
  }

  if (tr === undefined) {
    return null;
  }

  tr = tr.trim();

  tr = tr.split('=');

  if (tr.length < 2) {
    return null;
  }

  tr = tr[1].trim().split('-');

  if (tr.length !== 2) return null;

  var float = (tr[0].indexOf('.') > -1);
  var tseek = (tr[0].indexOf(':') > -1) ? 1 : 0;

  var start = parseTimeRange(tr[0].trim(), 0);
  var end   = parseTimeRange(tr[1].trim(), duration);

  return {start: start, end: end, duration: duration, float: float, tseek: tseek};

}

exports.getTimeRange = getTimeRange;

//-----------------------------------------------------------------------------

function parseTimeRange(val, def) {

  if (val.length === 0) return def;

  if (val.indexOf(':') < 0) {
    val = parseFloat(val);
    return (Number.isFinite(val)) ? (val * 1000) : def;
  }

  var t = val.split(':');

  if (t.length !== 3) return def;

  var hh = parseInt(t[0]);
  var mm = parseInt(t[1]);
  var ss = parseFloat(t[2]);

  if (Number.isFinite(hh) && Number.isFinite(mm) && Number.isFinite(ss)) {
    return ((ss * 1000) + (mm * 60 * 1000) + (hh * 60 * 60 * 1000));
  }

  return def;

}

//-----------------------------------------------------------------------------

var tpad = fmt.dig2;
var mpad = fmt.dig3;

function fmtTimeRange(d, float) {
  var t =  tpad(d.getUTCHours()) + ':' + tpad(d.getUTCMinutes()) + ':' + tpad(d.getUTCSeconds());
  if (float) t += '.' + mpad(d.getUTCMilliseconds());
  return t;
}

function fmtTimeLong(d, float) {
  var t = (d.getUTCHours() * 60 * 60) + (d.getUTCMinutes() * 60) + d.getUTCSeconds();
  if (float) t += '.' + mpad(d.getUTCMilliseconds());
  return t;
}

//-----------------------------------------------------------------------------

function setTimeRange(res, dev, tr) {

  if (tr === null || tr === undefined) return;

  var npt = 'npt=';

  var s = new Date(tr.start);
  var e = new Date(tr.end);
  var d = new Date(tr.duration);

  var proc = (dev.tseek === 1 || tr.tseek === 1) ? fmtTimeRange : fmtTimeLong;

  npt += proc(s, tr.float) + '-' + proc(e) + '/' + proc(d);

  res.setHeader(TIMESEEK_RANGE, npt);
  res.setHeader(XSEEK_RANGE, npt);

}

exports.setTimeRange = setTimeRange;