/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

var fs = require("fs");
var os = require("os");
var utl = require("util");

var arg = require("./arg");
var fmt = require("./fmt");

var ENC = arg.stdEncoding();

var LOG_TRACE = 1;
var LOG_DEBUG = 2;
var LOG_INFO  = 3;
var LOG_WARN  = 4;
var LOG_ERROR = 5;
var LOG_FATAL = 6;
var LOG_ALL   = 7;

var LEVELS = {
  trace: LOG_TRACE,
  debug: LOG_DEBUG,
  info: LOG_INFO,
  warn: LOG_WARN,
  error: LOG_ERROR,
  fatal: LOG_FATAL,
  all:   LOG_ALL
};

exports.LEVELS = LEVELS;

var LOG_SSDP  = (1 << 0);
var LOG_SOAP  = (1 << 1);
var LOG_HTTP  = (1 << 2);
var LOG_CODER = (1 << 3);

var names = [null, ' [TRC] ', ' [DBG] ', ' [INF] ', ' [WRN] ', ' [ERR] ', ' [FATAL] ', ' [ALL] '];

var level = LOG_TRACE;
var parts = 0; // ssdp | soap | http
var logtty = process.stdout.isTTY;
var stream = null;
var lines  = [];

function log(lvl, args, name) {

  var line = fmt.timeStamp(new Date());

  if (name === undefined) {
    name = names[lvl];
  }

  line += name;

  if (args.length > 1) {
    line += utl.format.apply(null, args);
  }
  else {
    line += args[0];
  }

  line += os.EOL;

  if (logtty) {
    var std = (lvl > LOG_WARN) ? process.stderr : process.stdout;
    try { std.write(line, ENC); } catch (e) { }
  }

  if (stream) {
    stream.write(line);
  }
  else if (lines && lines.length < 1000) {
    lines.push([lvl, line]);
  }

}

//-----------------------------------------------------------------------------
// Stream Setup
//-----------------------------------------------------------------------------

function onStreamError(err) {
  stream = null;
  lines  = null;
  if (!err) err = 'Error in log stream';
  try { process.stderr.write(err.toString() + '\n', ENC); } catch (e) { }
}

exports.initLogFile = function(lpath) {
  stream = fs.createWriteStream(lpath, {flags: 'w', encoding: 'utf8'});
  stream.once('error', onStreamError);
  if (lines) {
    for (var i = 0; i < lines.length; i++) {
      var ll = lines[i];
      if (ll[0] >= level) {
        stream.write(ll[1]);
      }
    }
    lines = null;
  }
};

exports.end = function(cb) {

  if (stream) {
    var tmp = stream;
    stream = null;
    tmp.end(cb);
  }
  else {
    cb();
  }

};

//-----------------------------------------------------------------------------
// Exports
//-----------------------------------------------------------------------------

exports.trace = function() { if (LOG_TRACE >= level) log(LOG_TRACE, arguments) };
exports.debug = function() { if (LOG_DEBUG >= level) log(LOG_DEBUG, arguments) };
exports.info = function() { if (LOG_INFO >= level) log(LOG_INFO, arguments) };
exports.warn = function() { if (LOG_WARN >= level) log(LOG_WARN, arguments) };
exports.error = function() { if (LOG_ERROR >= level) log(LOG_ERROR, arguments) };
exports.fatal = function() { log(LOG_FATAL, arguments) };
exports.all   = function() { log(LOG_ALL, arguments) };

exports.getLevel = function() { return level };
exports.setLevel = function(logLevel) { level = logLevel };

exports.canDebug = function() { return (LOG_DEBUG >= level) };

//-----------------------------------------------------------------------------
// LOG PARTS
//-----------------------------------------------------------------------------

exports.setParts = function(logParts) {
  parts = logParts;
};

exports.ssdp  = function() { if ((parts & LOG_SSDP)) log(LOG_INFO, arguments, ' [SSDP] ') };
exports.soap  = function() { if ((parts & LOG_SOAP)) log(LOG_INFO, arguments, ' [SOAP] ') };
exports.http  = function() { if ((parts & LOG_HTTP)) log(LOG_INFO, arguments, ' [HTTP] ') };
exports.coder = function() { if ((parts & LOG_CODER)) log(LOG_INFO, arguments, ' [ENC] ') };

exports.canSsdp  = function() { return ((parts & LOG_SSDP)) };
exports.canHttp  = function() { return ((parts & LOG_HTTP)) };
exports.canSoap  = function() { return ((parts & LOG_SOAP)) };
exports.canCoder = function() { return ((parts & LOG_CODER)) };

//-----------------------------------------------------------------------------

exports.border = function(text) {
  return '----------------------------------------------------------------\n' +
    text +
    '\n----------------------------------------------------------------';
};

//-----------------------------------------------------------------------------

exports.stringify = function(obj) {
  return prettyJSON(obj);
};

exports.millisPeriod = function(millisStart) {
  var millis = (Date.now() - millisStart) / 1000.0;
  return millis.toFixed(3);
};

//-----------------------------------------------------------------------------
// Console Redirects for third party modules
//-----------------------------------------------------------------------------

console.log = function() { log(LOG_DEBUG, arguments) };
console.info = function() { log(LOG_INFO, arguments) };
console.warn = function() { log(LOG_WARN, arguments) };
console.error = function() { log(LOG_ERROR, arguments) };

//-----------------------------------------------------------------------------
// Deprecation hook
//-----------------------------------------------------------------------------

process.on('deprecation', function(err) {
  var msg = (err['stack']) ? err['stack'] : err.toString();
  log(LOG_ERROR, [msg]);
});

//-----------------------------------------------------------------------------
// Formatters
//-----------------------------------------------------------------------------

var MAX_INDENT = 64;

function prettyJSON(val) {
  var buf = {str: ''};
  prettyValue(buf, val, 0);
  return buf.str;
}

function prettyString(buf, str, indent) {
  var len = str.length;
  if (len === 0) {
    buf.str += '""';
    return;
  }
  if (str.charAt(0) === '<') {
    buf.str += '\n' + pp.xml(str, indent);
    return;
  }
//  if (len > 60 && str.indexOf(',') > 0) {
//    var ary = str.split(',');
//    var prefix = INDENTS[indent] + '  ';
//    buf.str += '\n' + prefix;
//    buf.str += ary.join(',\n' + prefix);
//  }
  buf.str += str;
}

function prettyValue(buf, val, indent) {

  if (val === undefined) {
    buf.str += 'undefined';
    return;
  }

  if (val === null) {
    buf.str += 'null';
    return;
  }

  var tof = typeof val;

  switch (tof) {
    case 'string':
      prettyString(buf, val, indent);
      return;
    case 'number':
    case 'boolean':
      buf.str += val;
      return;
  }

  if (Array.isArray(val)) {
    prettyArray(buf, val, indent);
    return;
  }

  if (val instanceof Date) {
    buf.str += val.toUTCString();
    return;
  }

  if (val instanceof Error) {
    buf.str += val.toString();
    return;
  }

  prettyObject(buf, val, indent);

}

function prettyArray(buf, ary, indent) {
  var len = ary.length;
  if (len === 0) {
    buf.str += '[]';
    return;
  }
  indent += 2;
  if (indent > MAX_INDENT) {
    buf.str += '[......]';
    return;
  }
  buf.str += '[\n';
  for (var i = 0; i < len; i++) {
    if (i > 0) buf.str += ',\n';
    buf.str += INDENTS[indent];
    prettyValue(buf, ary[i], indent);
  }
  buf.str += '\n]';
}

function prettyObject(buf, obj, indent) {
  var keys = Object.keys(obj);
  var len = keys.length;
  if (len === 0) {
    buf.str += '{}';
    return;
  }
  indent += 2;
  if (indent > MAX_INDENT) {
    buf.str += '{......}';
    return;
  }
  buf.str += '{\n';
  for (var i = 0; i < len; i++) {
    if (i > 0) buf.str += ',\n';
    var key = keys[i];
    buf.str += INDENTS[indent] + key + ': ';
    prettyValue(buf, obj[key], indent);
  }
  buf.str += '\n}';
}

//-----------------------------------------------------------------------------

var INDENTS = [];

function initIndents() {
  var buf = '';
  for (var i = 0; i < MAX_INDENT; i += 2) {
    INDENTS[i] = buf;
    buf += '  ';
  }
}

initIndents();

/******************************************************************************
 The following code authored and licensed by:
 Copyright (c) 2012 Vadim Kiryukhin ( vkiryukhin @ gmail.com )
 Home page: http://www.eslinstructor.net/pretty-data/
 License: Dual licensed under the MIT and GPL licenses:
 http://www.opensource.org/licenses/mit-license.php
******************************************************************************/

function PP() {
  this.shift = ['\n']; // array of shifts
  this.step = '  '; // 2 spaces
  var maxdeep = 100, // nesting level
    ix = 0;

  // initialize array with shifts //
  for (ix = 0; ix < maxdeep; ix++) {
    this.shift.push(this.shift[ix] + this.step);
  }

}

PP.prototype.xml = function(text, indent) {

  var ar = text.replace(/>\s{0,}</g, "><")
      .replace(/</g, "~::~<")
      .replace(/xmlns\:/g, "~::~xmlns:")
      .replace(/xmlns\=/g, "~::~xmlns=")
      .split('~::~'),
    len = ar.length,
    inComment = false,
    deep = indent || 0,
    str = '',
    ix = 0;

  for (ix = 0; ix < len; ix++) {
    // start comment or <![CDATA[...]]> or <!DOCTYPE //
    if (ar[ix].search(/<!/) > -1) {
      str += this.shift[deep] + ar[ix];
      inComment = true;
      // end comment  or <![CDATA[...]]> //
      if (ar[ix].search(/-->/) > -1 || ar[ix].search(/\]>/) > -1 || ar[ix].search(/!DOCTYPE/) > -1) {
        inComment = false;
      }
    } else
    // end comment  or <![CDATA[...]]> //
    if (ar[ix].search(/-->/) > -1 || ar[ix].search(/\]>/) > -1) {
      str += ar[ix];
      inComment = false;
    } else
    // <elm></elm> //
    if (/^<\w/.exec(ar[ix - 1]) && /^<\/\w/.exec(ar[ix]) &&
      /^<[\w:\-\.\,]+/.exec(ar[ix - 1]) == /^<\/[\w:\-\.\,]+/.exec(ar[ix])[0].replace('/', '')) {
      str += ar[ix];
      if (!inComment) deep--;
    } else
    // <elm> //
    if (ar[ix].search(/<\w/) > -1 && ar[ix].search(/<\//) == -1 && ar[ix].search(/\/>/) == -1) {
      str = !inComment ? str += this.shift[deep++] + ar[ix] : str += ar[ix];
    } else
    // <elm>...</elm> //
    if (ar[ix].search(/<\w/) > -1 && ar[ix].search(/<\//) > -1) {
      str = !inComment ? str += this.shift[deep] + ar[ix] : str += ar[ix];
    } else
    // </elm> //
    if (ar[ix].search(/<\//) > -1) {
      str = !inComment ? str += this.shift[--deep] + ar[ix] : str += ar[ix];
    } else
    // <elm/> //
    if (ar[ix].search(/\/>/) > -1) {
      str = !inComment ? str += this.shift[deep] + ar[ix] : str += ar[ix];
    } else
    // <? xml ... ?> //
    if (ar[ix].search(/<\?/) > -1) {
      str += this.shift[deep] + ar[ix];
    } else
    // xmlns //
    if (ar[ix].search(/xmlns\:/) > -1 || ar[ix].search(/xmlns\=/) > -1) {
      str += this.shift[deep] + ar[ix];
    }

    else {
      str += ar[ix];
    }
  }

  return  (str[0] == '\n') ? str.slice(1) : str;

};

var pp = new PP();

/******************************************************************************
  END OF THIRD-PARTY CODE
******************************************************************************/

