/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Media Data Runtime Type Information

(function(exports) {

  // Media Object Properties

  var id = {
    uid:   'uid',   // Unique Resource ID
    error: 'error', // File processing error info
    fpath: 'fpath', // Full file/folder path
    flink: 'flink', // Real file/folder path for links
    fname: 'fname', // File/folder name without extension
    fextn: 'fextn', // File extension, not used for folders
    ftype: 'ftype', // File type (see below)
    fmode: 'fmode', // File/folder FS attributes
    fmime: 'fmime', // File mime type
    fsize: 'fsize', // File size
    froot: 'froot', // Top folder path
    mtime: 'mtime', // File/folder modification time in millis
    ctime: 'ctime', // File/folder creation time in millis
    mtype: 'mtype', // File media type (see below)
    index: 'index', // Curent image or PDF index
    fhidn: 'fhidn', // File/folder hidden
    minfo: 'minfo', // MediaInfo Text
    video: 'video', // Video Tracks
    audio: 'audio', // Audio Tracks
    text:  'text',  // Subtitles Tracks
    // Image Related
    orn:    'orn',    // Orientation
    bpp:    'bpp',    // Bits per Pixel
    clr:    'clr',    // Color Model
    idxclr: 'idxclr', // Indexed color model
    width:  'width',  // Width
    height: 'height', // Height
    count:  'count',  // Number of sub images or PDF pages
    // Thumb Related
    tvalid:  'tvalid',  // Has valid thumbnail
    tpath:   'tpath',   // alternative thumb path
    twidth:  'twidth',  // Thumb width
    theight: 'theight',  // Thumb height
    // Meta Data
    title: 'title',
    // Top Folder
    istop: 'istop',
    // Lack modified check stamp
    stamp: 'stamp',
    // AV Keys
    videocount: 'videocount',
    audiocount: 'audiocount',
    textcount:  'textcount',
    duration:   'duration',
    format:     'format',
    channels:   'channels',
    codec:      'codec',
    codecid:    'codecid',
    bitrate:    'bitrate',
    bitmode:    'bitmode',
    scantype:   'scantype', // Progressive | Interlaced
    language:   'language',
    valid:      'valid', // track valid flag
    samplingrate: 'samplingrate',
    framerate:   'framerate',
    streamorder: 'streamorder',
    dsr: 'displayaspectratio',
    subflag: 'subflag', // sub files loaded
    // Meta Data
    album:      'album',
    track:      'track',
    genre:      'genre',
    performer:  'performer'
  };

  exports.id = id;

  // File Type

  var ftype = {
    unknown: 0,
    file:    1,
    folder:  2
  };

  exports.ftype = ftype;

  // Media Type

  var mtype = {
    unknown: 0,
    image:   1,
    audio:   2,
    video:   4,
    pdf:     8,
    text:    16
  };

  exports.mtype = mtype;

  // Device Properties

  exports.did = {
    addr: 'addr',
    uuid: 'uuid',
    hash: 'hash',
    face: 'face',
    from: 'from',
    href: 'href',   // SSDP Location
    hmap: 'hmap',   // HTTP headers
    umap: 'umap',   // USN map (updated with notify)
    smap: 'smap',   // Services map (from description)
    alias: 'alias', // Device Name Alias
    ready: 'ready', // Device XML Loaded
    icons: 'icons', // icons map
    ctime: 'ctime', // Created
    mtime: 'mtime', // Updated / Modified
    atime: 'atime', // Last access time from device
    ua:    'ua',    // User Agent
    friendlyName: 'friendlyName',
    manufacturer: 'manufacturer',
    modelName: 'modelName',
    modelDescription: 'modelDescription',
    modelNumber: 'modelNumber',
    modelURL: 'modelURL',
    manufacturerURL: 'manufacturerURL',
    deviceType: 'deviceType',
    UDN: 'UDN',
    path: 'path',
    serviceType: 'serviceType',
    // Device Config
    dcfg:   'dcfg',
    title:  'title',
    sreso:  'sreso',
    tvmode: 'tvmode' // 0 - PAL, 1 - NTSC
  };

  // Folder sort

  var sort = {
    name:   0,
    newest: 1,
    oldest: 2
  };

  exports.sort = sort;

  // Type Detectors

  exports.isFile = function(md) {
    return (md[id.ftype] === ftype.file);
  };

  exports.isFolder = function(md) {
    return (md[id.ftype] === ftype.folder);
  };

  exports.isUnknownFileType = function(md) {
    return (md[id.ftype] === ftype.unknown);
  };

  exports.isHidden = function(md) {
    return md[id.fhidn];
  };

  exports.isUnknownMediaType = function(md) {
    return md[id.mtype] === mtype.unknown;
  };

  exports.isImage = function(md) {
    return md[id.mtype] === mtype.image;
  };

  exports.isAudio = function(md) {
    return md[id.mtype] === mtype.audio;
  };

  exports.isVideo = function(md) {
    return md[id.mtype] === mtype.video;
  };

  exports.isPDF = function(md) {
    return md[id.mtype] === mtype.pdf;
  };

  exports.isImageOrPDF = function(md) {
    var mt = md[id.mtype];
    return (mt === mtype.image || mt === mtype.pdf);
  };

  // Media Data Helpers

  exports.uid = function(md) {
    return md[id.uid];
  };

  exports.fpath = function(md) {
    return md[id.flink] || md[id.fpath];
  };

  exports.fmime = function(md) {
    return md[id.fmime];
  };

  exports.format = function(md) {
    return md[id.codecid] || md[id.codec] || md[id.format];
  };

  exports.codecid = function(md) {
    return md[id.codecid] || md[id.codec] || md[id.format];
  };

  exports.codec = function(md) {
    return md[id.codec] || md[id.format];
  };

  exports.duration = function(md) {
    return md[id.duration];
  };

  exports.fname = function(md, hideExt) {
    var fname = md[id.fname];
    if (!hideExt) {
      var ext = md[id.fextn];
      if (ext !== undefined && ext !== null) {
        fname += '.' + ext;
      }
    }
    return fname;
  };

  exports.hasDuration = function(md) {
    var dur = md[id.duration];
    return (typeof dur === 'number' && !isNaN(dur));
  };

  exports.videoCount = function(md) {
    var ary = md[id.video];
    return (Array.isArray(ary)) ? ary.length : 0;
  };

  exports.audioCount = function(md) {
    var ary = md[id.audio];
    return (Array.isArray(ary)) ? ary.length : 0;
  };

  exports.textCount = function(md) {
    var ary = md[id.text];
    return (Array.isArray(ary)) ? ary.length : 0;
  };

  exports.videoTrack = function(md, index) {
    if (index === undefined) index = 0;
    return md[id.video][index];
  };

  exports.audioTrack = function(md, index) {
    if (index === undefined) index = 0;
    return md[id.audio][index];
  };

  exports.textTrack = function(md, index) {
    if (index === undefined) index = 0;
    return md[id.text][index];
  };

  exports.audioLangIndex = function(md, lang) {
    var ac = exports.audioCount(md);
    if (ac === 0) return -2;
    var aa = md[id.audio];
    for (var i = 0; i < ac; i++) {
      var aud = aa[i];
      var ary = aud[id.language];
      if (ary && ary.indexOf(lang) > -1) {
        return i;
      }
    }
    return -1;
  };

  exports.audioLangName = function(aud, def) {
    var ary = aud[id.language];
    return (ary) ? ary[0] : def;
  };

  exports.textLangName = function(sub, def) {
    var ary = sub[id.language];
    return (ary) ? ary[0] : def;
  };

  exports.textLangIndex = function(md, lang) {
    var sc = exports.textCount(md);
    if (sc === 0) return -2;
    var sa = md[id.text];
    for (var i = 0; i < sc; i++) {
      var sub = sa[i];
      var ary = sub[id.language];
      if (ary && ary.indexOf(lang) > -1) {
        return i;
      }
    }
    return -1;
  };

  exports.isInterlaced = function(vid) {
    return (vid && vid[id.scantype] === 'Interlaced');
  };

  exports.frameRate = function(md) {
    return md[id.framerate] || 0;
  };

  exports.getDSR = function(vid) {
    return vid[id.dsr];
  };

  exports.width = function(obj) {
    return obj[id.width];
  };

  exports.streamIndex = function(stream) {
    return stream[id.streamorder];
  };

  exports.channels = function(aud) {
    return aud[id.channels];
  };

  exports.getSubFlag = function(md) {
    return md[id.subflag];
  };

  exports.setSubFlag = function(md) {
    md[id.subflag] = true;
  };

  exports.getSubTracks = function(md) {
    var ary = md[id.text];
    if (!ary) {
      ary = [];
      md[id.text] = ary;
    }
    return ary;
  };

  // Thumbnail

  exports.isThumbValid = function(md) {
    return md[id.tvalid];
  };

  exports.thumbWidth = function(md) {
    return md[id.twidth];
  };

  exports.thumbHeight = function(md) {
    return md[id.theight];
  };

  // Generic Type Helpers

  exports.notNull = function(val) {
    return (val !== undefined && val !== null);
  };

  exports.isNull = function(val) {
    return (val === undefined || val === null);
  };

  exports.isObject = function(arg) {
    return (typeof arg === 'object' && arg !== null && !Array.isArray(arg));
  };

  exports.isArray = function(arg) {
    return Array.isArray(arg);
  };

  exports.isFunction = function(arg) {
    return typeof arg === 'function';
  };

  exports.isNumber = function(arg) {
    return (typeof arg === 'number' && !isNaN(arg));
  };

  exports.isString = function(arg) {
    return (typeof arg === 'string');
  };

  function objectToString(obj) {
    return Object.prototype.toString.call(obj);
  }

})(typeof exports === 'undefined'? this['rti'] = {} : exports);