/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

// Remote Device Registry

'use strict';

var request = require('request');
var async = require('async');
var nu = require('url');

var upnp = require('../cls/upnp');

var evt = require('../evt');
var log = require('../log');
var rti = require('../rti');
var acl = require('../acl');
var fmt = require('../fmt');
var cfg = require('../cfg');
var md5 = require('../md5');
var xml = require('../xml');
var htp = require('../htp');
var ifa = require('../ifa');
var jsl = require('../jsl');
var ping = require('../sys').ping;

var id  = rti.did;

var dmap = {}; // Address -> Device
var cmap = {}; // Address -> Device config map
var xmap = {}; // UUID - > XML descriptions
var chld = {}; // Child Devices

var DEF_RESO = [1920, 1080];

var DEF_DCFG = {
  title: 'Generic Device',
  alias: null,
  sreso: DEF_RESO,
  acode: 'lpcm-full',
  vcode: 'mp2v',
  tseek: 0,
  tvs: 0,     // TV system
  chn: 6,     // Audio channels
  asr: 48000, // audio sampling rate
  arts: [160, 160],
  artp: "JPEG_TN",
  fmt: [
    ['a', 'mpeg audio']
  ],
  mime: {}
};

//-----------------------------------------------------------------------------
// Public Methods
//-----------------------------------------------------------------------------

exports.getDevices = function(hidden) {
  var ary = [];
  for (var addr in dmap) {
    var dev = dmap[addr];
    if (dev[id.ready] && (!isHidden(dev) || hidden)) {
      ary.push(dev);
    }
  }
  return ary;
};

exports.hasDevice = function(uuid) {
  return (findDev(uuid) !== null);
};

exports.getDevice = function(uuid) {
  return findDev(uuid);
};

exports.getXML = function(uuid) {
  return xmap[uuid];
};

exports.refresh = function(cb) {

  var keys = Object.keys(dmap);

  function refreshKey(addr, rcb) {
    var dev = dmap[addr];
    if (dev && dev[id.ready]) {
      if (dev[id.href]) {
        loadDevice(dev, true, true, rcb);
      }
      else {
        ping(dev[id.addr], function(err) {
          if (err) {
            removeDevice(dev[id.uuid], true);
          }
          rcb();
        });
      }
    }
    else {
      rcb();
    }
  }

  async.each(keys, refreshKey, cb);

};

exports.getDeviceConfig = function(addr) {
  var dcfg = cmap[addr];
  if (dcfg) {
    return dcfg;
  }
  log.warn('Device config not found for request from: %s', addr);
  cmap[addr] = DEF_DCFG;
  return DEF_DCFG;
};

exports.updateConfig = function(uuid, dcfg) {
  var dev = findDev(uuid);
  if (dev) {
    dev[id.dcfg] = dcfg;
    setConfigMap(dev[id.addr], dcfg);
    evt.postDevUpdate(dev);
  }
};

exports.getDefCfg = function() {
  return DEF_DCFG;
};

//-----------------------------------------------------------------------------
// Helpers
//-----------------------------------------------------------------------------

function isHidden(dev) {
  return (cfg.devHidden(dev[id.uuid]) || cfg.devHidden(dev[id.deviceType]));
}

function setConfigMap(addr, dcfg) {
  cmap[addr] = dcfg;
}

function getDev(addr) {
  return dmap[addr];
}

function setDev(dev) {
  var addr = dev[id.addr];
  dmap[addr] = dev;
  cmap[addr] = dev[id.dcfg];
}

function findDev(uuid) {
  for (var addr in dmap) {
    var dev = dmap[addr];
    if (dev[id.uuid] === uuid) {
      return dev;
    }
  }
  return null;
}

//-----------------------------------------------------------------------------
// Create Device
//-----------------------------------------------------------------------------

function createSsdpDevice(uuid, msg) {

  var href = msg.href;

  if (!href) {
    log.warn('Device SSDP message without LOCATION:\n%s', fmt.border(msg.text));
    return;
  }

  var dev = {};

  var addr = msg.from.addr;
  var from = jsl.copyValue(msg.from);
  delete from['size'];

  dev[id.addr]  = addr;
  dev[id.uuid]  = uuid;
  dev[id.hash]  = md5(uuid);
  dev[id.face]  = msg.face;
  dev[id.from]  = from;
  dev[id.href]  = href;
  dev[id.hmap]  = {};
  dev[id.ctime] = msg.when;
  dev[id.mtime] = msg.when;
  dev[id.icons] = {};
  dev[id.ready] = false;

  var umap = {};
  umap[uuid] = [msg.usn];
  dev[id.umap] = umap;

  dev[id.smap] = {};

  loadDevice(dev);

}

//-----------------------------------------------------------------------------
// Remove Device
//-----------------------------------------------------------------------------

function removeDevice(uuid, emitEvent) {
  var dev = findDev(uuid);
  if (dev) {
    var addr = dev[id.addr];
    delete dmap[addr];
    delete xmap[uuid];
    if (emitEvent) {
      evt.postDevRemove(dev);
    }
  }
}

//-----------------------------------------------------------------------------
// Load Device
//-----------------------------------------------------------------------------

function loadDevice(dev, update, remove, cb) {

  var href = dev[id.href];
  var uuid = dev[id.uuid];

  request({url: href, timeout: 4000}, function(err, res, body) {
    if (err) {
      if (!remove) {
        log.error('Error loading device info from: %s (%s)', href, err);
      }
      if (remove) removeDevice(uuid, update);
      if (cb) cb();
      return;
    }
    if (res.statusCode !== 200) {
      if (!remove) {
        log.error('Status %d for loading device info from: %s', res.statusCode, href);
      }
      if (remove) removeDevice(uuid, update);
      if (cb) cb();
      return;
    }
    if (update && xmap[uuid] === body) {
      if (cb) cb();
      return;
    }
    xmap[uuid] = body;
    parseDeviceInfo(dev, body, update);
    if (cb) cb();
  });

}

//-----------------------------------------------------------------------------
// Parse Device Info
//-----------------------------------------------------------------------------

function parseDeviceInfo(dev, body, update) {


  var $ = xml.parseXML(body, true);

  $('UDN').each(function(i) {
    var uuid = fmt.uuidFromUDN($(this).text().trim());
    if (uuid) {
      if (i > 0) {
        if (!update) {
          log.debug('Child device found: %s', uuid);
        }
        chld[uuid] = true;
      }
      var umap = dev[id.umap];
      if (!umap[uuid]) {
        umap[uuid] = [];
      }
    }
  });

  dev[id.friendlyName]    = xtext($, 'friendlyName');
  dev[id.manufacturer]    = xtext($, 'manufacturer');
  dev[id.modelURL]        = xtext($, 'modelURL');
  dev[id.manufacturerURL] = xtext($, 'manufacturerURL');
  dev[id.deviceType]      = xtext($, 'deviceType');

  parseDeviceIcon($, dev);

  parseDeviceServices($, dev);

  dev[id.ready] = true;
  dev[id.dcfg]  = createDevConfig(dev);

  setDev(dev);

  if (!update) {
    evt.postDevCreate(dev, $);
  }
  else {
    evt.postDevUpdate(dev);
  }

}

function xtext($, name) {
  return $(name).eq(0).text().trim();
}

function xtrim(elm) {
  return elm.text().trim();
}

function parseDeviceIcon($, dev) {
  $('icon').each(function() {
    var ico = $(this);
    var mime = xtrim(ico.children('mimetype'));
    if (mime) {
      var w = xtrim(ico.children('width'));
      var h = xtrim(ico.children('height'));
      var u = xtrim(ico.children('url'));
      if (w && h && u) {
        u = nu.resolve(dev[id.href], u);
        var icons = dev[id.icons];
        var micon = icons[mime];
        if (!micon) {
          micon = {};
          icons[mime] = micon;
        }
        micon[String(w)] = {w: w, h: h, u: u};
      }
    }
  });
}

function parseDeviceServices($, dev) {
  $('service').each(function() {
    var svc = $(this);
    var type = xtrim(svc.children('serviceType'));
    if (type) {
      var uctl = xtrim(svc.children('controlURL'));
      var uevt = xtrim(svc.children('eventSubURL'));
      var uscp = xtrim(svc.children('SCPDURL'));
      var href = dev[id.href];
      if (uctl) uctl = nu.resolve(href, uctl);
      if (uevt) uevt = nu.resolve(href, uevt);
      if (uscp) uscp = nu.resolve(href, uscp);
      dev[id.smap][type] = {uctl: uctl, uevt: uevt, uscp: uscp};
    }
  });
}

//-----------------------------------------------------------------------------
// Update Device
//-----------------------------------------------------------------------------

function updateSsdpDevice(uuid, dev, msg) {
  dev[id.mtime] = msg.when;
  var umap = dev[id.umap];
  var uary = umap[uuid];
  if (uary === undefined) {
    uary = [];
    umap[uuid] = uary;
  }
  var usn = msg.usn;
  if (usn && uary.indexOf(usn) === -1) {
    uary.push(usn);
  }
  if (usn === upnp.ROOT_DEVICE) {
    loadDevice(dev, true, false);
  }
}

function shouldRemove(uuid, dev, msg) {
  var umap = dev[id.umap];
  var uary = umap[uuid];
  if (!uary || uary.length === 0) return true;
  var ix = uary.indexOf(msg.usn);
  if (ix > -1) {
    uary.splice(ix, 1);
  }
  return (uary.length === 0);
}

//-----------------------------------------------------------------------------
// SSDP Hook
//-----------------------------------------------------------------------------

function onSsdpMessage(msg) {

  if (msg.isSearch()) return;

  var uuid = msg.uuid;

  if (!uuid) {
    log.warn('Device SSDP message without UUID:\n%s', fmt.border(msg.text));
    return;
  }

  if (chld[uuid]) return;

  var dev = findDev(uuid);

  if (msg.isByeBye()) {
    if (dev && shouldRemove(uuid, dev, msg)) {
      removeDevice(uuid, true);
    }
    return;
  }

  if (dev && !dev.href) {
    dev = null;
  }

  if (dev) {
    if (!isHidden(dev)) {
      updateSsdpDevice(uuid, dev, msg);
    }
  }
  else if (msg.usn === upnp.ROOT_DEVICE) {
    createSsdpDevice(uuid, msg);
  }

}

evt.onSsdpMessage(onSsdpMessage);

//-----------------------------------------------------------------------------
// Security Check
//-----------------------------------------------------------------------------

function onConfigChange() {
  var keys = Object.keys(dmap);
  for (var i = 0; i < keys.length; i++) {
    var addr = keys[i];
    var dev = dmap[addr];
    if (!acl.checkSSDP(dev[id.face], dev[id.from])) {
      removeDevice(dev[id.uuid], true);
    }
  }
}

evt.onConfigChange(onConfigChange);

//-----------------------------------------------------------------------------
// NetFace Hook
//-----------------------------------------------------------------------------

function onNetFaceOff(face) {
  var keys = Object.keys(dmap);
  for (var i = 0; i < keys.length; i++) {
    var addr = keys[i];
    var dev = dmap[addr];
    if (dev.face.addr === face.addr) {
      removeDevice(dev[id.uuid], true);
    }
  }
}

evt.onNetFaceOff(onNetFaceOff);

//-----------------------------------------------------------------------------
// Loggers
//-----------------------------------------------------------------------------

evt.onDevCreate(function(dev) {
  var addr = dev[id.addr];
  log.info('Device created "%s" from: %s', dev[id.friendlyName] || addr, dev[id.href] || addr);
  if (!isHidden(dev)) {
    evt.postNotifyUser(dev[id.friendlyName] + ' connected');
  }
});

evt.onDevRemove(function(dev) {
  log.info('Device removed "%s"', dev[id.friendlyName] || dev[id.addr]);
  if (!isHidden(dev)) {
    evt.postNotifyUser(dev[id.friendlyName] + ' removed');
  }
});

evt.onDevUpdate(function(dev) {
  updateDevConfig(dev);
  log.info('Device updated "%s"', dev[id.friendlyName] || dev[id.addr]);
});

//-----------------------------------------------------------------------------
// Device Configuration
//-----------------------------------------------------------------------------

function createDevConfig(dev) {
  var uuid = dev[id.uuid];
  var dcfg = cfg.getDevCfg(uuid);
  if (dcfg) {
    if (dcfg[id.title] !== dev[id.friendlyName]) {
      dcfg[id.title] = dev[id.friendlyName];
      cfg.setDevCfg(uuid, dcfg);
    }
    jsl.mergeObject(DEF_DCFG, dcfg);
    evt.postDevDefaults(dcfg);
    return dcfg;
  }
  dcfg = jsl.copyValue(DEF_DCFG);
  dcfg[id.title] = dev[id.friendlyName];
  evt.postDevDefaults(dcfg);
  cfg.setDevCfg(uuid, dcfg);
  return dcfg;
}

function updateDevConfig(dev) {
  var uuid = dev[id.uuid];
  var dcfg = cfg.getDevCfg(uuid);
  if (!dcfg) {
    log.warn('Device updated without config object: %s', dev[id.friendlyName]);
    return;
  }
  var save = false;
  if (dcfg[id.title] !== dev[id.friendlyName]) {
    dcfg[id.title] = dev[id.friendlyName];
    save = true;
  }
  if (save) {
    dev[id.dcfg] = dcfg;
    setConfigMap(dev[id.addr], dcfg);
    cfg.setDevCfg(uuid, dcfg);
  }
}

//-----------------------------------------------------------------------------
// HTTP Device Management
//-----------------------------------------------------------------------------

function onSoapRequest(req) {
  var dev = getDev(req.ip);
  if (dev && !dev[id.ua]) {
    var ua = htp.userAgent(req);
    if (ua) {
      dev[id.ua] = ua;
      evt.postDevUpdate(dev);
    }
  }
}

function onBeforeBrowseRequest(req, arg) {

  var addr = req.ip;

  var dev = getDev(addr);

  if (dev && dev[id.href]) { // Device is present and is a valid UPnP implementor
    return;
  }

  var xv = htp.getXAV(req);
  var ua = htp.userAgent(req);

  if (!xv) {
    xv = {};
    xv[id.friendlyName] = ua || req.ip;
  }

  var friendlyName = xv[id.friendlyName] || 'Unknown Device';
  var manufacturer = xv[id.manufacturer] || 'Unknown Manufacturer';

  var uuid = md5.uuid(friendlyName + manufacturer);

  if (dev && dev[id.uuid] === uuid) { // Device is present
    dev[id.mtime] = Date.now();
    return;
  }

  var ladr = req.socket.localAddress;
  var face = ifa.getFace(ladr);

  if (!face) {
    log.warn('Unable to find local network interface for: %s', ladr);
    return;
  }

  dev = {};

  dev[id.addr]  = addr;
  dev[id.uuid]  = uuid;
  dev[id.hash]  = md5(uuid);
  dev[id.face]  = face;
  dev[id.from]  = {addr: addr, port: req.socket.remotePort};
  dev[id.href]  = null;
  dev[id.hmap]  = {};
  dev[id.ctime] = Date.now();
  dev[id.mtime] = Date.now();
  dev[id.icons] = {};
  dev[id.ready] = true;
  dev[id.umap]  = {};
  dev[id.smap]  = {};
  dev[id.ua]    = ua;
  dev[id.friendlyName] = friendlyName;
  dev[id.manufacturer] = manufacturer;
  dev[id.deviceType]   = 'urn:schemas-upnp-org:device:Basic:1';
  dev[id.dcfg] = createDevConfig(dev);

  setDev(dev);

  evt.postDevCreate(dev, null);

}

evt.onSoapRequest(onSoapRequest);
evt.onBeforeBrowseRequest(onBeforeBrowseRequest);

evt.onInit(function() {
  evt.postDevDefaults(DEF_DCFG);
});