/*
 * Copyright (c) 2014 iSedora.com - All Rights Reserved
 * This file is part of iSedora Media Server software
 */

'use strict';

var cheerio = require('cheerio');

var PROLOG = '<?xml version="1.0" encoding="UTF-8"?>';

var NS_STRIP_RX = new RegExp('<(/?)\\w+:', 'g');

var CHEERIO_XML_OPT = { normalizeWhitespace: true, xmlMode: true };

// new-school escaping

function esc(str) {
  return String(str)
    .replace(/&/g, '&amp;')
    .replace(/"/g, '&quot;')
    .replace(/'/g, '&#39;')
    .replace(/</g, '&lt;')
    .replace(/>/g, '&gt;');
}

function getVal(val) {
  var tof = typeof val;
  if (tof === 'string') {
    return esc(val);
  }
  else if (tof === 'number' || tof === 'boolean') {
    return val.toString();
  }
  else if (tof === 'function') {
    return getVal(val());
  }
  else {
    throw "TAG value is not a primitive of function";
  }
}

function build(tags, parentIndex, parentName, obj, lev) {

  var allkeys = Object.keys(obj);

  var klen = allkeys.length;

  if (klen === 0) {
    if (parentName === null) {
      throw "parent name is null";
    }
    tags[parentIndex] += '></' + parentName + '>';
    return klen;
  }

  var kidx, key, val, list, llen, lidx, tof, tagIndex, childCount;

  for(kidx = 0; kidx < klen; kidx++) {

    key = allkeys[kidx];

    if (key === undefined) {
      throw 'Undefined key at index: ' + kidx + ' of len: ' + klen;
    }

    if (key === '@') {
      if (parentIndex === -1) {
        throw 'parent index is invalid for attr: ' + key;
      }
      var aval = obj[key];
      for (var akey in aval) {
        tags[parentIndex] += ' ' + akey + '="' + getVal(aval[akey]) + '"';
      }
    }
    else if (key === '#text') {
      tags.push(getVal(obj[key]));
    }
    else if (key === '#list') {
      list = obj[key];
      if (!Array.isArray(list)) {
        throw '#list value is not an Array';
      }
      llen = list.length;
      for (lidx = 0; lidx < llen; lidx++) {
        build(tags, -1, null, list[lidx], lev);
      }
    }
    else {
      tags.push('<' + key);
      tagIndex = tags.length - 1;
      val = obj[key];
      if (val === null) {
        tags[tagIndex] += '/>';
      }
      else {
        tof = typeof val;
        if (tof === 'string') {
          tags[tagIndex] += '>' + esc(val) + '</' + key + '>';
        }
        else if (tof === 'number' || tof === 'boolean') {
          tags[tagIndex] += '>' + val + '</' + key + '>';
        }
        else if (tof === 'function') {
          tags[tagIndex] += '>' + getVal(val()) + '</' + key + '>';
        }
        else if (tof === 'object') {
          childCount = 0;
          if (Array.isArray(val)) {
            childCount = val.length;
            for (lidx = 0; lidx < childCount; lidx++) {
              build(tags, -1, null, val[lidx], lev + 1);
            }
          }
          else {
            childCount = build(tags, tagIndex, key, val, lev + 1);
          }
          if (childCount) {
            tags[tagIndex] += '>';
            tags.push('</' + key + '>');
          }
          else {
            tags[tagIndex] += '/>';
          }
        }
        else {
          throw 'invalid tag value: ' + key;
        }
      }
    }

  }

  return klen;

}

exports.streamJSON = function(res, obj, prolog) {
  prolog = prolog || PROLOG;
  var tags = [prolog];
  build(tags, -1, null, obj, 0);
  res.send(tags.join(''));
//  var doc = xb.create(obj);
//  res.send(doc.end(opts));
};

function toXML(obj, prolog) {
  var tags = [];
  if (prolog === undefined || prolog === true) {
    prolog = PROLOG;
  }
  if (prolog) {
    tags.push(prolog);
  }
  build(tags, -1, null, obj, 0);
  return tags.join('');
}

exports.toXML = toXML;

//-----------------------------------------------------------------------------
// Parsing
//-----------------------------------------------------------------------------

function parseXML(text, stripNS) {
  if (stripNS) {
    text = text.replace(NS_STRIP_RX, '<$1');
  }
  return cheerio.load(text, CHEERIO_XML_OPT);
}

exports.parseXML = parseXML;

//-----------------------------------------------------------------------------
// SOAP Action
//-----------------------------------------------------------------------------

function parseSoapAction(text, args) {

  if (!text) return undefined;

  var $ = parseXML(text, true);

  var tact = $('Body').children().first();

  if (tact.length === 0) return undefined;

  tact.children().each(function() {
    args[this.name.toLowerCase()] = $(this).text().trim();
  });

  return tact.get(0).name;

}

exports.parseSoapAction = parseSoapAction;

//-----------------------------------------------------------------------------
// SOAP Response
//-----------------------------------------------------------------------------

function createSoapResponse(action, namespace, response) {

  var soap = {
    's:Envelope': {
      '@': {
        'xmlns:s': 'http://schemas.xmlsoap.org/soap/envelope/',
        's:encodingStyle': 'http://schemas.xmlsoap.org/soap/encoding/'
      },
      's:Body': {}
    }
  };

  var rtag = 'u:' + action + 'Response';

  response['@'] = { 'xmlns:u': namespace };

  soap['s:Envelope']['s:Body'][rtag] = response;

  return toXML(soap, false);

}

exports.createSoapResponse = createSoapResponse;

//-----------------------------------------------------------------------------
// UPnP Fault
//-----------------------------------------------------------------------------

function createUpnpFault(err) {

  var fault = {
    's:Envelope': {
      '@': {
        'xmlns:s': 'http://schemas.xmlsoap.org/soap/envelope/',
        's:encodingStyle': 'http://schemas.xmlsoap.org/soap/encoding/'
      },
      's:Body': {
        's:Fault': {
          'faultcode': 's:Client',
          'faultstring': 'UPnPError',
          'detail': {
            'UPnPError': {
              '@': { 'xmlns': 'urn:schemas-upnp-org:control-1-0' },
              'errorCode': 501,
              'errorDescription': err.toString()
            }
          }
        }
      }
    }
  };

  return toXML(fault, false);

}

exports.createUpnpFault = createUpnpFault;

//-----------------------------------------------------------------------------
// Event Properties
//-----------------------------------------------------------------------------

function eventProperties(obj, caller) {

  var prop = {};

  for (var key in obj) {
    var val = obj[key];
    if (typeof val === 'function') {
      val = val(caller);
    }
    prop[key] = val;
  }

  var pset = {
    'e:propertyset': {
      '@': { 'xmlns:e': "urn:schemas-upnp-org:event-1-0"},
      'e:property': prop
    }
  };

  return toXML(pset, false);

}

exports.eventProperties = eventProperties;
